"""Forem tap class."""

from __future__ import annotations

from singer_sdk import Stream, Tap
from singer_sdk import typing as th

from tap_forem.streams import Articles, Comments

__all__ = ["TapForem"]

STREAM_TYPES = [
    Articles,
    Comments,
]


class TapForem(Tap):
    """Singer tap for the Forem API."""

    name = "tap-forem"

    config_jsonschema = th.PropertiesList(
        th.Property(
            "api_key",
            th.StringType,
            required=True,
            description="The Forem API key.",
        ),
        th.Property(
            "tag",
            th.StringType,
            required=True,
            description="Tag for filter articles by.",
        ),
        th.Property(
            "api_url",
            th.StringType,
            default="https://dev.to/api/",
            description="The url for the API service.",
        ),
    ).to_dict()

    def discover_streams(self) -> list[Stream]:
        """Return a list of discovered streams."""
        return [stream_class(tap=self) for stream_class in STREAM_TYPES]



"""REST client handling, including ForemStream base class."""

from __future__ import annotations

import typing as t
from pathlib import Path

from singer_sdk.authenticators import APIKeyAuthenticator
from singer_sdk.pagination import BasePageNumberPaginator
from singer_sdk.streams import RESTStream

if t.TYPE_CHECKING:
    from singer_sdk.helpers.types import Context

SCHEMAS_DIR = Path(__file__).parent / Path("./schemas")


class ForemStream(RESTStream[int]):
    """Forem stream class."""

    records_jsonpath = "$[*]"

    @property
    def url_base(self) -> str:
        """Return the API URL root, configurable via tap settings."""
        return self.config["api_url"]  # type: ignore[no-any-return]

    @property
    def authenticator(self) -> APIKeyAuthenticator:
        """Return a new authenticator object."""
        return APIKeyAuthenticator.create_for_stream(
            self,
            key="api-key",
            value=self.config["api_key"],
            location="header",
        )

    @property
    def http_headers(self) -> dict[str, str]:
        """Return the http headers needed."""
        return {
            "User-Agent": f"{self.tap_name}/{self._tap.plugin_version}",
            "Accept": "application/vnd.forem.api-v1+json",
        }


class PaginatedForemStream(ForemStream):
    """Forem stream with pagination."""

    def get_new_paginator(self) -> BasePageNumberPaginator:
        """Get a fresh paginator.

        Returns:
            A new paginator for the Forem API.
        """
        return BasePageNumberPaginator(1)

    def get_url_params(
        self,
        context: Context | None,  # noqa: ARG002
        next_page_token: t.Any | None,  # noqa: ANN401
    ) -> dict[str, t.Any]:
        """Return a dictionary of values to be used in URL parameterization."""
        return {
            "page": next_page_token,
            "per_page": self._page_size,
        }



"""Stream type classes for tap-forem."""

from __future__ import annotations

import typing as t

from singer_sdk import typing as th

from tap_forem.client import ForemStream, PaginatedForemStream

if t.TYPE_CHECKING:
    from singer_sdk.helpers.types import Context

USER_TYPE = th.ObjectType(
    th.Property("name", th.StringType),
    th.Property("username", th.StringType),
    th.Property("twitter_username", th.StringType),
    th.Property("github_username", th.StringType),
    th.Property("website_url", th.StringType),
    th.Property("profile_image", th.StringType),
    th.Property("profile_image_90", th.StringType),
    th.Property("user_id", th.IntegerType),
)

ORG_TYPE = th.ObjectType(
    th.Property("name", th.StringType),
    th.Property("username", th.StringType),
    th.Property("slug", th.StringType),
    th.Property("profile_image", th.StringType),
    th.Property("profile_image_90", th.StringType),
)

FLARE_TAG_TYPE = th.ObjectType(
    th.Property("name", th.StringType),
    th.Property("bg_color_hex", th.StringType),
    th.Property("text_color_hex", th.StringType),
)


class Articles(PaginatedForemStream):
    """Articles stream."""

    name = "articles"
    path = "/articles"
    primary_keys = ("id",)

    schema = th.PropertiesList(
        th.Property(
            "id",
            th.IntegerType,
            description="The article's system ID",
            required=True,
        ),
        th.Property("type_of", th.StringType),
        th.Property("title", th.StringType),
        th.Property("description", th.StringType),
        th.Property("cover_image", th.StringType),
        th.Property("social_image", th.StringType),
        th.Property("city", th.StringType),
        th.Property("slug", th.StringType),
        th.Property("path", th.StringType),
        th.Property("url", th.StringType),
        th.Property("canonical_url", th.StringType),
        th.Property("created_at", th.DateTimeType),
        th.Property("edited_at", th.DateTimeType),
        th.Property("crossposted_at", th.DateTimeType),
        th.Property("published_at", th.DateTimeType),
        th.Property("last_comment_at", th.DateTimeType),
        th.Property("tag_list", th.ArrayType(th.StringType)),
        th.Property("tags", th.StringType),
        th.Property("collection_id", th.IntegerType),
        th.Property("reading_time_minutes", th.IntegerType),
        th.Property("comments_count", th.IntegerType),
        th.Property("positive_reactions_count", th.IntegerType),
        th.Property("public_reactions_count", th.IntegerType),
        th.Property("page_views_count", th.IntegerType),
        th.Property("user", USER_TYPE),
        th.Property("organization", ORG_TYPE),
        th.Property("flare_tag", FLARE_TAG_TYPE),
        th.Property("readable_publish_date", th.StringType),
        th.Property("published_timestamp", th.DateTimeType),
    ).to_dict()

    def get_url_params(
        self,
        context: Context | None,
        next_page_token: t.Any | None,  # noqa: ANN401
    ) -> dict[str, t.Any]:
        """Get query parameters."""
        params = super().get_url_params(context, next_page_token)
        params["tag"] = self.config.get("tag")
        return params

    def get_child_context(
        self,
        record: dict[str, t.Any],
        context: Context | None,  # noqa: ARG002
    ) -> dict[str, t.Any]:
        """Get context for article children."""
        return {"article_id": record["id"], "comments_count": record["comments_count"]}


class Comments(ForemStream):
    """Comments stream."""

    name = "comments"
    path = "/comments"
    primary_keys = ("id_code",)
    parent_stream_type = Articles

    schema = th.PropertiesList(
        th.Property(
            "id_code",
            th.StringType,
            description="The comment's system ID",
            required=True,
        ),
        th.Property("type_of", th.StringType),
        th.Property("created_at", th.DateTimeType),
        th.Property("article_id", th.IntegerType),
        th.Property("body_html", th.StringType),
        th.Property("children", th.ArrayType(th.ObjectType())),
        th.Property("user", USER_TYPE),
    ).to_dict()

    def get_url_params(
        self,
        context: Context | None,
        next_page_token: t.Any | None,  # noqa: ANN401
    ) -> dict[str, t.Any] | str:
        """Get query parameters."""
        params = super().get_url_params(context, next_page_token)
        if context:
            params["a_id"] = context["article_id"]  # type: ignore[index]
        return params

    def should_sync(self, context: Context | None) -> bool:
        """Sync comments only if article has any."""
        return bool(context and context["comments_count"] > 0)



"""Tests standard tap features using the built-in SDK tests library."""

from __future__ import annotations

from singer_sdk.testing import get_tap_test_class

from tap_forem.tap import TapForem

SAMPLE_CONFIG = {
    "tag": "sentry",
}

TestTapForem = get_tap_test_class(TapForem, config=SAMPLE_CONFIG)
